import { ApolloProviderProviderConsumer } from "../../utils/provider";
export class ApolloQueryComponent {
    componentWillLoad() {
        this.startSubscription();
    }
    onPropsChange() {
        this.stopSubscription();
        this.startSubscription();
    }
    componentDidUnload() {
        this.stopSubscription();
    }
    getResult() {
        return {
            data: this.originalResult && this.originalResult.data,
            loading: this.originalResult ? this.originalResult.loading : true,
            error: {
                graphQLErrors: this.originalResult && this.originalResult.errors,
                networkError: undefined,
                message: this.originalResult && this.originalResult.errors && this.originalResult.errors[0] && this.originalResult.errors[0].message,
                name: this.originalResult && this.originalResult.errors && this.originalResult.errors[0] && this.originalResult.errors[0].name,
                extraInfo: this.originalResult && this.originalResult.errors && this.originalResult.errors[0] && this.originalResult.errors[0].originalError,
            },
            variables: this.variables,
            networkStatus: this.originalResult && this.originalResult.networkStatus,
            refetch: this.observable && this.observable.refetch.bind(this.observable),
            fetchMore: this.observable && this.observable.fetchMore.bind(this.observable),
            startPolling: this.observable && this.observable.startPolling.bind(this.observable),
            stopPolling: this.observable && this.observable.stopPolling.bind(this.observable),
            subscribeToMore: this.observable && this.observable.subscribeToMore.bind(this.observable),
            updateQuery: this.observable && this.observable.updateQuery.bind(this.observable),
            client: this.client,
        };
    }
    startSubscription() {
        if (this.client) {
            const observable = this.client.watchQuery(Object.assign({ query: this.query, variables: this.variables }, this.options));
            this._subscription = observable.subscribe(originalResult => {
                this.originalResult = originalResult;
                this.resultEventEmitter.emit(this.getResult());
            }, error => {
                this.error = error;
                this.resultEventEmitter.emit(this.getResult());
            });
            this.readyEventEmitter.emit(this.getResult());
        }
        else {
            throw new Error('You should wrap your parent component with apollo-provider custom element or ApolloProvider functional component');
        }
    }
    stopSubscription() {
        if (this._subscription) {
            this._subscription.unsubscribe();
        }
    }
    render() {
        return this.renderer && this.renderer(this.getResult());
    }
    static get is() { return "apollo-query"; }
    static get properties() { return {
        "client": {
            "type": "Any",
            "attr": "client",
            "watchCallbacks": ["onPropsChange"]
        },
        "el": {
            "elementRef": true
        },
        "error": {
            "state": true
        },
        "options": {
            "type": "Any",
            "attr": "options",
            "watchCallbacks": ["onPropsChange"]
        },
        "originalResult": {
            "state": true
        },
        "query": {
            "type": "Any",
            "attr": "query",
            "watchCallbacks": ["onPropsChange"]
        },
        "renderer": {
            "type": "Any",
            "attr": "renderer",
            "watchCallbacks": ["onPropsChange"]
        },
        "variables": {
            "type": "Any",
            "attr": "variables",
            "watchCallbacks": ["onPropsChange"]
        }
    }; }
    static get events() { return [{
            "name": "ready",
            "method": "readyEventEmitter",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "result",
            "method": "resultEventEmitter",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
}
ApolloProviderProviderConsumer.injectProps(ApolloQueryComponent, ['client']);
