import { ApolloProviderProviderConsumer } from "../../utils/provider";
export class ApolloSubscriptionComponent {
    componentWillLoad() {
        this.result = {
            data: undefined,
            errors: [],
            loading: true,
            networkStatus: undefined,
            stale: undefined
        };
        this.startSubscription();
    }
    onPropsChange() {
        this.stopSubscription();
        this.startSubscription();
    }
    componentDidUnload() {
        this.stopSubscription();
    }
    startSubscription() {
        if (this.client) {
            this._subscription = this.client.subscribe(Object.assign({ query: this.subscription, variables: this.variables }, this.options)).subscribe(result => {
                this.result = result;
                this.result.emit(this.result);
            });
            this.readyEventEmitter.emit(this.result);
        }
        else {
            throw new Error('You should wrap your parent component with apollo-provider custom element or ApolloProvider functional component');
        }
    }
    stopSubscription() {
        if (this._subscription) {
            this._subscription.unsubscribe();
        }
    }
    render() {
        return this.renderer && this.renderer(this.result);
    }
    static get is() { return "apollo-subscription"; }
    static get properties() { return {
        "client": {
            "type": "Any",
            "attr": "client",
            "watchCallbacks": ["onPropsChange"]
        },
        "el": {
            "elementRef": true
        },
        "options": {
            "type": "Any",
            "attr": "options",
            "watchCallbacks": ["onPropsChange"]
        },
        "renderer": {
            "type": "Any",
            "attr": "renderer",
            "watchCallbacks": ["onPropsChange"]
        },
        "result": {
            "state": true
        },
        "subscription": {
            "type": "Any",
            "attr": "subscription",
            "watchCallbacks": ["onPropsChange"]
        },
        "variables": {
            "type": "Any",
            "attr": "variables",
            "watchCallbacks": ["onPropsChange"]
        }
    }; }
    static get events() { return [{
            "name": "ready",
            "method": "readyEventEmitter",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "result",
            "method": "resultEventEmitter",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
}
ApolloProviderProviderConsumer.injectProps(ApolloSubscriptionComponent, ['client']);
